"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .order import Order, OrderTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookOrderRefundedPayloadTypedDict(TypedDict):
    r"""Sent when an order is fully or partially refunded.

    **Discord & Slack support:** Full
    """

    timestamp: datetime
    data: OrderTypedDict
    type: Literal["order.refunded"]


class WebhookOrderRefundedPayload(BaseModel):
    r"""Sent when an order is fully or partially refunded.

    **Discord & Slack support:** Full
    """

    timestamp: datetime

    data: Order

    TYPE: Annotated[
        Annotated[
            Literal["order.refunded"], AfterValidator(validate_const("order.refunded"))
        ],
        pydantic.Field(alias="type"),
    ] = "order.refunded"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionUpdateTrialTypedDict(TypedDict):
    trial_end: datetime
    r"""Set or extend the trial period of the subscription. If set to `now`, the trial will end immediately."""


class SubscriptionUpdateTrial(BaseModel):
    trial_end: datetime
    r"""Set or extend the trial period of the subscription. If set to `now`, the trial will end immediately."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerseat import CustomerSeat, CustomerSeatTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class SeatsListTypedDict(TypedDict):
    seats: List[CustomerSeatTypedDict]
    r"""List of seats"""
    available_seats: int
    r"""Number of available seats"""
    total_seats: int
    r"""Total number of seats for the subscription"""


class SeatsList(BaseModel):
    seats: List[CustomerSeat]
    r"""List of seats"""

    available_seats: int
    r"""Number of available seats"""

    total_seats: int
    r"""Total number of seats for the subscription"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_webhookdelivery_ import (
    ListResourceWebhookDelivery,
    ListResourceWebhookDeliveryTypedDict,
)
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


EndpointIDTypedDict = TypeAliasType("EndpointIDTypedDict", Union[str, List[str]])
r"""Filter by webhook endpoint ID."""


EndpointID = TypeAliasType("EndpointID", Union[str, List[str]])
r"""Filter by webhook endpoint ID."""


class WebhooksListWebhookDeliveriesRequestTypedDict(TypedDict):
    endpoint_id: NotRequired[Nullable[EndpointIDTypedDict]]
    r"""Filter by webhook endpoint ID."""
    start_timestamp: NotRequired[Nullable[datetime]]
    r"""Filter deliveries after this timestamp."""
    end_timestamp: NotRequired[Nullable[datetime]]
    r"""Filter deliveries before this timestamp."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""


class WebhooksListWebhookDeliveriesRequest(BaseModel):
    endpoint_id: Annotated[
        OptionalNullable[EndpointID],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by webhook endpoint ID."""

    start_timestamp: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter deliveries after this timestamp."""

    end_timestamp: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter deliveries before this timestamp."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "endpoint_id",
            "start_timestamp",
            "end_timestamp",
            "page",
            "limit",
        ]
        nullable_fields = ["endpoint_id", "start_timestamp", "end_timestamp"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class WebhooksListWebhookDeliveriesResponseTypedDict(TypedDict):
    result: ListResourceWebhookDeliveryTypedDict


class WebhooksListWebhookDeliveriesResponse(BaseModel):
    next: Callable[[], Optional[WebhooksListWebhookDeliveriesResponse]]

    result: ListResourceWebhookDelivery



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .aggregationfunction import AggregationFunction
from .benefitsortproperty import BenefitSortProperty
from .benefittype import BenefitType
from .customersortproperty import CustomerSortProperty
from .eventsortproperty import EventSortProperty
from .eventsource import EventSource
from .listresource_benefit_ import ListResourceBenefit, ListResourceBenefitTypedDict
from .listresource_customer_ import ListResourceCustomer, ListResourceCustomerTypedDict
from .listresource_event_ import ListResourceEvent, ListResourceEventTypedDict
from .listresource_meter_ import ListResourceMeter, ListResourceMeterTypedDict
from .listresource_order_ import ListResourceOrder, ListResourceOrderTypedDict
from .listresource_product_ import ListResourceProduct, ListResourceProductTypedDict
from .listresource_subscription_ import (
    ListResourceSubscription,
    ListResourceSubscriptionTypedDict,
)
from .metersortproperty import MeterSortProperty
from .ordersortproperty import OrderSortProperty
from .productbillingtype import ProductBillingType
from .productsortproperty import ProductSortProperty
from .subscriptionsortproperty import SubscriptionSortProperty
from .timeinterval import TimeInterval
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Callable, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OrganizationIDFilterTypedDict = TypeAliasType(
    "OrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


OrganizationIDFilter = TypeAliasType("OrganizationIDFilter", Union[str, List[str]])
r"""Filter by organization ID."""


ProductIDFilterTypedDict = TypeAliasType(
    "ProductIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by product ID."""


ProductIDFilter = TypeAliasType("ProductIDFilter", Union[str, List[str]])
r"""Filter by product ID."""


CustomerIDFilterTypedDict = TypeAliasType(
    "CustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


CustomerIDFilter = TypeAliasType("CustomerIDFilter", Union[str, List[str]])
r"""Filter by customer ID."""


ExternalCustomerIDFilterTypedDict = TypeAliasType(
    "ExternalCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer external ID."""


ExternalCustomerIDFilter = TypeAliasType(
    "ExternalCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer external ID."""


DiscountIDFilterTypedDict = TypeAliasType(
    "DiscountIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by discount ID."""


DiscountIDFilter = TypeAliasType("DiscountIDFilter", Union[str, List[str]])
r"""Filter by discount ID."""


MetadataQueryTypedDict = TypeAliasType(
    "MetadataQueryTypedDict", Union[str, int, bool, List[str], List[int], List[bool]]
)


MetadataQuery = TypeAliasType(
    "MetadataQuery", Union[str, int, bool, List[str], List[int], List[bool]]
)


class SubscriptionsListRequestTypedDict(TypedDict):
    organization_id: NotRequired[Nullable[OrganizationIDFilterTypedDict]]
    r"""Filter by organization ID."""
    product_id: NotRequired[Nullable[ProductIDFilterTypedDict]]
    r"""Filter by product ID."""
    customer_id: NotRequired[Nullable[CustomerIDFilterTypedDict]]
    r"""Filter by customer ID."""
    external_customer_id: NotRequired[Nullable[ExternalCustomerIDFilterTypedDict]]
    r"""Filter by customer external ID."""
    discount_id: NotRequired[Nullable[DiscountIDFilterTypedDict]]
    r"""Filter by discount ID."""
    active: NotRequired[Nullable[bool]]
    r"""Filter by active or inactive subscription."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[SubscriptionSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class SubscriptionsListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[OrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    product_id: Annotated[
        OptionalNullable[ProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    customer_id: Annotated[
        OptionalNullable[CustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    external_customer_id: Annotated[
        OptionalNullable[ExternalCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer external ID."""

    discount_id: Annotated[
        OptionalNullable[DiscountIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by discount ID."""

    active: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by active or inactive subscription."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[SubscriptionSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "product_id",
            "customer_id",
            "external_customer_id",
            "discount_id",
            "active",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = [
            "organization_id",
            "product_id",
            "customer_id",
            "external_customer_id",
            "discount_id",
            "active",
            "sorting",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class SubscriptionsListResponseTypedDict(TypedDict):
    result: ListResourceSubscriptionTypedDict


class SubscriptionsListResponse(BaseModel):
    next: Callable[[], Optional[SubscriptionsListResponse]]

    result: ListResourceSubscription


QueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "QueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


QueryParamOrganizationIDFilter = TypeAliasType(
    "QueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


BenefitTypeFilterTypedDict = TypeAliasType(
    "BenefitTypeFilterTypedDict", Union[BenefitType, List[BenefitType]]
)
r"""Filter by benefit type."""


BenefitTypeFilter = TypeAliasType(
    "BenefitTypeFilter", Union[BenefitType, List[BenefitType]]
)
r"""Filter by benefit type."""


class BenefitsListRequestTypedDict(TypedDict):
    organization_id: NotRequired[Nullable[QueryParamOrganizationIDFilterTypedDict]]
    r"""Filter by organization ID."""
    type_filter: NotRequired[Nullable[BenefitTypeFilterTypedDict]]
    r"""Filter by benefit type."""
    query: NotRequired[Nullable[str]]
    r"""Filter by description."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[BenefitSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class BenefitsListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[QueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    type_filter: Annotated[
        OptionalNullable[BenefitTypeFilter],
        pydantic.Field(alias="type"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by benefit type."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by description."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[BenefitSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "type_filter",
            "query",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = [
            "organization_id",
            "type_filter",
            "query",
            "sorting",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class BenefitsListResponseTypedDict(TypedDict):
    result: ListResourceBenefitTypedDict


class BenefitsListResponse(BaseModel):
    next: Callable[[], Optional[BenefitsListResponse]]

    result: ListResourceBenefit


QueryParamProductIDFilterTypedDict = TypeAliasType(
    "QueryParamProductIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by product ID."""


QueryParamProductIDFilter = TypeAliasType(
    "QueryParamProductIDFilter", Union[str, List[str]]
)
r"""Filter by product ID."""


ProductsListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "ProductsListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


ProductsListQueryParamOrganizationIDFilter = TypeAliasType(
    "ProductsListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


BenefitIDFilterTypedDict = TypeAliasType(
    "BenefitIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter products granting specific benefit."""


BenefitIDFilter = TypeAliasType("BenefitIDFilter", Union[str, List[str]])
r"""Filter products granting specific benefit."""


class ProductsListRequestTypedDict(TypedDict):
    id: NotRequired[Nullable[QueryParamProductIDFilterTypedDict]]
    r"""Filter by product ID."""
    organization_id: NotRequired[
        Nullable[ProductsListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    query: NotRequired[Nullable[str]]
    r"""Filter by product name."""
    is_archived: NotRequired[Nullable[bool]]
    r"""Filter on archived products."""
    is_recurring: NotRequired[Nullable[bool]]
    r"""Filter on recurring products. If `true`, only subscriptions tiers are returned. If `false`, only one-time purchase products are returned."""
    benefit_id: NotRequired[Nullable[BenefitIDFilterTypedDict]]
    r"""Filter products granting specific benefit."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[ProductSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class ProductsListRequest(BaseModel):
    id: Annotated[
        OptionalNullable[QueryParamProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    organization_id: Annotated[
        OptionalNullable[ProductsListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product name."""

    is_archived: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter on archived products."""

    is_recurring: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter on recurring products. If `true`, only subscriptions tiers are returned. If `false`, only one-time purchase products are returned."""

    benefit_id: Annotated[
        OptionalNullable[BenefitIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter products granting specific benefit."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[ProductSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "organization_id",
            "query",
            "is_archived",
            "is_recurring",
            "benefit_id",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = [
            "id",
            "organization_id",
            "query",
            "is_archived",
            "is_recurring",
            "benefit_id",
            "sorting",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ProductsListResponseTypedDict(TypedDict):
    result: ListResourceProductTypedDict


class ProductsListResponse(BaseModel):
    next: Callable[[], Optional[ProductsListResponse]]

    result: ListResourceProduct


OrdersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "OrdersListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


OrdersListQueryParamOrganizationIDFilter = TypeAliasType(
    "OrdersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


OrdersListQueryParamProductIDFilterTypedDict = TypeAliasType(
    "OrdersListQueryParamProductIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by product ID."""


OrdersListQueryParamProductIDFilter = TypeAliasType(
    "OrdersListQueryParamProductIDFilter", Union[str, List[str]]
)
r"""Filter by product ID."""


ProductBillingTypeFilterTypedDict = TypeAliasType(
    "ProductBillingTypeFilterTypedDict",
    Union[ProductBillingType, List[ProductBillingType]],
)
r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


ProductBillingTypeFilter = TypeAliasType(
    "ProductBillingTypeFilter", Union[ProductBillingType, List[ProductBillingType]]
)
r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


QueryParamDiscountIDFilterTypedDict = TypeAliasType(
    "QueryParamDiscountIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by discount ID."""


QueryParamDiscountIDFilter = TypeAliasType(
    "QueryParamDiscountIDFilter", Union[str, List[str]]
)
r"""Filter by discount ID."""


OrdersListQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "OrdersListQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


OrdersListQueryParamCustomerIDFilter = TypeAliasType(
    "OrdersListQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


CheckoutIDFilterTypedDict = TypeAliasType(
    "CheckoutIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by checkout ID."""


CheckoutIDFilter = TypeAliasType("CheckoutIDFilter", Union[str, List[str]])
r"""Filter by checkout ID."""


class OrdersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[OrdersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    product_id: NotRequired[Nullable[OrdersListQueryParamProductIDFilterTypedDict]]
    r"""Filter by product ID."""
    product_billing_type: NotRequired[Nullable[ProductBillingTypeFilterTypedDict]]
    r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""
    discount_id: NotRequired[Nullable[QueryParamDiscountIDFilterTypedDict]]
    r"""Filter by discount ID."""
    customer_id: NotRequired[Nullable[OrdersListQueryParamCustomerIDFilterTypedDict]]
    r"""Filter by customer ID."""
    checkout_id: NotRequired[Nullable[CheckoutIDFilterTypedDict]]
    r"""Filter by checkout ID."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[OrderSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class OrdersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[OrdersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    product_id: Annotated[
        OptionalNullable[OrdersListQueryParamProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    product_billing_type: Annotated[
        OptionalNullable[ProductBillingTypeFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""

    discount_id: Annotated[
        OptionalNullable[QueryParamDiscountIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by discount ID."""

    customer_id: Annotated[
        OptionalNullable[OrdersListQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    checkout_id: Annotated[
        OptionalNullable[CheckoutIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by checkout ID."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[OrderSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "product_id",
            "product_billing_type",
            "discount_id",
            "customer_id",
            "checkout_id",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = [
            "organization_id",
            "product_id",
            "product_billing_type",
            "discount_id",
            "customer_id",
            "checkout_id",
            "sorting",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class OrdersListResponseTypedDict(TypedDict):
    result: ListResourceOrderTypedDict


class OrdersListResponse(BaseModel):
    next: Callable[[], Optional[OrdersListResponse]]

    result: ListResourceOrder


CustomersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "CustomersListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


CustomersListQueryParamOrganizationIDFilter = TypeAliasType(
    "CustomersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


class CustomersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[CustomersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    email: NotRequired[Nullable[str]]
    r"""Filter by exact email."""
    query: NotRequired[Nullable[str]]
    r"""Filter by name, email, or external ID."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[CustomerSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class CustomersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[CustomersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    email: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by exact email."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by name, email, or external ID."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[CustomerSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "email",
            "query",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = ["organization_id", "email", "query", "sorting", "metadata"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomersListResponseTypedDict(TypedDict):
    result: ListResourceCustomerTypedDict


class CustomersListResponse(BaseModel):
    next: Callable[[], Optional[CustomersListResponse]]

    result: ListResourceCustomer


EventsListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "EventsListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventsListQueryParamOrganizationIDFilter = TypeAliasType(
    "EventsListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventsListQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "EventsListQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


EventsListQueryParamCustomerIDFilter = TypeAliasType(
    "EventsListQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


QueryParamExternalCustomerIDFilterTypedDict = TypeAliasType(
    "QueryParamExternalCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by external customer ID."""


QueryParamExternalCustomerIDFilter = TypeAliasType(
    "QueryParamExternalCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by external customer ID."""


NameFilterTypedDict = TypeAliasType("NameFilterTypedDict", Union[str, List[str]])
r"""Filter by event name."""


NameFilter = TypeAliasType("NameFilter", Union[str, List[str]])
r"""Filter by event name."""


SourceFilterTypedDict = TypeAliasType(
    "SourceFilterTypedDict", Union[EventSource, List[EventSource]]
)
r"""Filter by event source."""


SourceFilter = TypeAliasType("SourceFilter", Union[EventSource, List[EventSource]])
r"""Filter by event source."""


class EventsListRequestTypedDict(TypedDict):
    filter_: NotRequired[Nullable[str]]
    r"""Filter events following filter clauses. JSON string following the same schema a meter filter clause."""
    start_timestamp: NotRequired[Nullable[datetime]]
    r"""Filter events after this timestamp."""
    end_timestamp: NotRequired[Nullable[datetime]]
    r"""Filter events before this timestamp."""
    organization_id: NotRequired[
        Nullable[EventsListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    customer_id: NotRequired[Nullable[EventsListQueryParamCustomerIDFilterTypedDict]]
    r"""Filter by customer ID."""
    external_customer_id: NotRequired[
        Nullable[QueryParamExternalCustomerIDFilterTypedDict]
    ]
    r"""Filter by external customer ID."""
    meter_id: NotRequired[Nullable[str]]
    r"""Filter by a meter filter clause."""
    name: NotRequired[Nullable[NameFilterTypedDict]]
    r"""Filter by event name."""
    source: NotRequired[Nullable[SourceFilterTypedDict]]
    r"""Filter by event source."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[EventSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class EventsListRequest(BaseModel):
    filter_: Annotated[
        OptionalNullable[str],
        pydantic.Field(alias="filter"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter events following filter clauses. JSON string following the same schema a meter filter clause."""

    start_timestamp: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter events after this timestamp."""

    end_timestamp: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter events before this timestamp."""

    organization_id: Annotated[
        OptionalNullable[EventsListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    customer_id: Annotated[
        OptionalNullable[EventsListQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    external_customer_id: Annotated[
        OptionalNullable[QueryParamExternalCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by external customer ID."""

    meter_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by a meter filter clause."""

    name: Annotated[
        OptionalNullable[NameFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by event name."""

    source: Annotated[
        OptionalNullable[SourceFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by event source."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[EventSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "filter",
            "start_timestamp",
            "end_timestamp",
            "organization_id",
            "customer_id",
            "external_customer_id",
            "meter_id",
            "name",
            "source",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = [
            "filter",
            "start_timestamp",
            "end_timestamp",
            "organization_id",
            "customer_id",
            "external_customer_id",
            "meter_id",
            "name",
            "source",
            "sorting",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EventsListResponseTypedDict(TypedDict):
    result: ListResourceEventTypedDict


class EventsListResponse(BaseModel):
    next: Callable[[], Optional[EventsListResponse]]

    result: ListResourceEvent


MetersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "MetersListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


MetersListQueryParamOrganizationIDFilter = TypeAliasType(
    "MetersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


class MetersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[MetersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    query: NotRequired[Nullable[str]]
    r"""Filter by name."""
    is_archived: NotRequired[Nullable[bool]]
    r"""Filter on archived meters."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[MeterSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class MetersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[MetersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by name."""

    is_archived: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter on archived meters."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[MeterSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "query",
            "is_archived",
            "page",
            "limit",
            "sorting",
            "metadata",
        ]
        nullable_fields = [
            "organization_id",
            "query",
            "is_archived",
            "sorting",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class MetersListResponseTypedDict(TypedDict):
    result: ListResourceMeterTypedDict


class MetersListResponse(BaseModel):
    next: Callable[[], Optional[MetersListResponse]]

    result: ListResourceMeter


MetersQuantitiesQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "MetersQuantitiesQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


MetersQuantitiesQueryParamCustomerIDFilter = TypeAliasType(
    "MetersQuantitiesQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


MetersQuantitiesQueryParamExternalCustomerIDFilterTypedDict = TypeAliasType(
    "MetersQuantitiesQueryParamExternalCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by external customer ID."""


MetersQuantitiesQueryParamExternalCustomerIDFilter = TypeAliasType(
    "MetersQuantitiesQueryParamExternalCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by external customer ID."""


class MetersQuantitiesRequestTypedDict(TypedDict):
    id: str
    r"""The meter ID."""
    start_timestamp: datetime
    r"""Start timestamp."""
    end_timestamp: datetime
    r"""End timestamp."""
    interval: TimeInterval
    r"""Interval between two timestamps."""
    customer_id: NotRequired[
        Nullable[MetersQuantitiesQueryParamCustomerIDFilterTypedDict]
    ]
    r"""Filter by customer ID."""
    external_customer_id: NotRequired[
        Nullable[MetersQuantitiesQueryParamExternalCustomerIDFilterTypedDict]
    ]
    r"""Filter by external customer ID."""
    customer_aggregation_function: NotRequired[Nullable[AggregationFunction]]
    r"""If set, will first compute the quantities per customer before aggregating them using the given function. If not set, the quantities will be aggregated across all events."""
    metadata: NotRequired[Nullable[Dict[str, MetadataQueryTypedDict]]]
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""


class MetersQuantitiesRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The meter ID."""

    start_timestamp: Annotated[
        datetime, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Start timestamp."""

    end_timestamp: Annotated[
        datetime, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""End timestamp."""

    interval: Annotated[
        TimeInterval,
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]
    r"""Interval between two timestamps."""

    customer_id: Annotated[
        OptionalNullable[MetersQuantitiesQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    external_customer_id: Annotated[
        OptionalNullable[MetersQuantitiesQueryParamExternalCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by external customer ID."""

    customer_aggregation_function: Annotated[
        OptionalNullable[AggregationFunction],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""If set, will first compute the quantities per customer before aggregating them using the given function. If not set, the quantities will be aggregated across all events."""

    metadata: Annotated[
        OptionalNullable[Dict[str, MetadataQuery]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = UNSET
    r"""Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "customer_id",
            "external_customer_id",
            "customer_aggregation_function",
            "metadata",
        ]
        nullable_fields = [
            "customer_id",
            "external_customer_id",
            "customer_aggregation_function",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrganizationNotificationSettingsTypedDict(TypedDict):
    new_order: bool
    new_subscription: bool


class OrganizationNotificationSettings(BaseModel):
    new_order: bool

    new_subscription: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookevent import WebhookEvent, WebhookEventTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class WebhookDeliveryTypedDict(TypedDict):
    r"""A webhook delivery for a webhook event."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    succeeded: bool
    r"""Whether the delivery was successful."""
    http_code: Nullable[int]
    r"""The HTTP code returned by the URL. `null` if the endpoint was unreachable."""
    response: Nullable[str]
    r"""The response body returned by the URL, or the error message if the endpoint was unreachable."""
    webhook_event: WebhookEventTypedDict
    r"""A webhook event.

    An event represent something that happened in the system
    that should be sent to the webhook endpoint.

    It can be delivered multiple times until it's marked as succeeded,
    each one creating a new delivery.
    """


class WebhookDelivery(BaseModel):
    r"""A webhook delivery for a webhook event."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    succeeded: bool
    r"""Whether the delivery was successful."""

    http_code: Nullable[int]
    r"""The HTTP code returned by the URL. `null` if the endpoint was unreachable."""

    response: Nullable[str]
    r"""The response body returned by the URL, or the error message if the endpoint was unreachable."""

    webhook_event: WebhookEvent
    r"""A webhook event.

    An event represent something that happened in the system
    that should be sent to the webhook endpoint.

    It can be delivered multiple times until it's marked as succeeded,
    each one creating a new delivery.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at", "http_code", "response"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .payment import Payment, PaymentTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceTypedDict(TypedDict):
    items: List[PaymentTypedDict]
    pagination: PaginationTypedDict


class ListResource(BaseModel):
    items: List[Payment]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerPaymentMethodConfirmTypedDict(TypedDict):
    setup_intent_id: str
    set_default: bool


class CustomerPaymentMethodConfirm(BaseModel):
    setup_intent_id: str

    set_default: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .seatstatus import SeatStatus
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict
from typing_extensions import NotRequired, TypedDict


class CustomerSeatTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The seat ID"""
    subscription_id: str
    r"""The subscription ID"""
    status: SeatStatus
    customer_id: NotRequired[Nullable[str]]
    r"""The assigned customer ID"""
    customer_email: NotRequired[Nullable[str]]
    r"""The assigned customer email"""
    invitation_token_expires_at: NotRequired[Nullable[datetime]]
    r"""When the invitation token expires"""
    claimed_at: NotRequired[Nullable[datetime]]
    r"""When the seat was claimed"""
    revoked_at: NotRequired[Nullable[datetime]]
    r"""When the seat was revoked"""
    seat_metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Additional metadata for the seat"""


class CustomerSeat(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The seat ID"""

    subscription_id: str
    r"""The subscription ID"""

    status: SeatStatus

    customer_id: OptionalNullable[str] = UNSET
    r"""The assigned customer ID"""

    customer_email: OptionalNullable[str] = UNSET
    r"""The assigned customer email"""

    invitation_token_expires_at: OptionalNullable[datetime] = UNSET
    r"""When the invitation token expires"""

    claimed_at: OptionalNullable[datetime] = UNSET
    r"""When the seat was claimed"""

    revoked_at: OptionalNullable[datetime] = UNSET
    r"""When the seat was revoked"""

    seat_metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Additional metadata for the seat"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "customer_id",
            "customer_email",
            "invitation_token_expires_at",
            "claimed_at",
            "revoked_at",
            "seat_metadata",
        ]
        nullable_fields = [
            "modified_at",
            "customer_id",
            "customer_email",
            "invitation_token_expires_at",
            "claimed_at",
            "revoked_at",
            "seat_metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .order import Order, OrderTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookOrderCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new order is created.

    A new order is created when:

    * A customer purchases a one-time product. In this case, `billing_reason` is set to `purchase`.
    * A customer starts a subscription. In this case, `billing_reason` is set to `subscription_create`.
    * A subscription is renewed. In this case, `billing_reason` is set to `subscription_cycle`.
    * A subscription is upgraded or downgraded with an immediate proration invoice. In this case, `billing_reason` is set to `subscription_update`.

    <Warning>The order might not be paid yet, so the `status` field might be `pending`.</Warning>

    **Discord & Slack support:** Full
    """

    timestamp: datetime
    data: OrderTypedDict
    type: Literal["order.created"]


class WebhookOrderCreatedPayload(BaseModel):
    r"""Sent when a new order is created.

    A new order is created when:

    * A customer purchases a one-time product. In this case, `billing_reason` is set to `purchase`.
    * A customer starts a subscription. In this case, `billing_reason` is set to `subscription_create`.
    * A subscription is renewed. In this case, `billing_reason` is set to `subscription_cycle`.
    * A subscription is upgraded or downgraded with an immediate proration invoice. In this case, `billing_reason` is set to `subscription_update`.

    <Warning>The order might not be paid yet, so the `status` field might be `pending`.</Warning>

    **Discord & Slack support:** Full
    """

    timestamp: datetime

    data: Order

    TYPE: Annotated[
        Annotated[
            Literal["order.created"], AfterValidator(validate_const("order.created"))
        ],
        pydantic.Field(alias="type"),
    ] = "order.created"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productcreateonetime import ProductCreateOneTime, ProductCreateOneTimeTypedDict
from .productcreaterecurring import (
    ProductCreateRecurring,
    ProductCreateRecurringTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ProductCreateTypedDict = TypeAliasType(
    "ProductCreateTypedDict",
    Union[ProductCreateOneTimeTypedDict, ProductCreateRecurringTypedDict],
)


ProductCreate = TypeAliasType(
    "ProductCreate", Union[ProductCreateOneTime, ProductCreateRecurring]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerpaymentmethodcreaterequiresactionresponse import (
    CustomerPaymentMethodCreateRequiresActionResponse,
    CustomerPaymentMethodCreateRequiresActionResponseTypedDict,
)
from .customerpaymentmethodcreatesucceededresponse import (
    CustomerPaymentMethodCreateSucceededResponse,
    CustomerPaymentMethodCreateSucceededResponseTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


CustomerPaymentMethodCreateResponseTypedDict = TypeAliasType(
    "CustomerPaymentMethodCreateResponseTypedDict",
    Union[
        CustomerPaymentMethodCreateSucceededResponseTypedDict,
        CustomerPaymentMethodCreateRequiresActionResponseTypedDict,
    ],
)


CustomerPaymentMethodCreateResponse = Annotated[
    Union[
        Annotated[
            CustomerPaymentMethodCreateRequiresActionResponse, Tag("requires_action")
        ],
        Annotated[CustomerPaymentMethodCreateSucceededResponse, Tag("succeeded")],
    ],
    Discriminator(lambda m: get_discriminator(m, "status", "status")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customer import Customer, CustomerTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookCustomerCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new customer is created.

    A customer can be created:

    * After a successful checkout.
    * Programmatically via the API.

    **Discord & Slack support:** Basic
    """

    timestamp: datetime
    data: CustomerTypedDict
    r"""A customer in an organization."""
    type: Literal["customer.created"]


class WebhookCustomerCreatedPayload(BaseModel):
    r"""Sent when a new customer is created.

    A customer can be created:

    * After a successful checkout.
    * Programmatically via the API.

    **Discord & Slack support:** Basic
    """

    timestamp: datetime

    data: Customer
    r"""A customer in an organization."""

    TYPE: Annotated[
        Annotated[
            Literal["customer.created"],
            AfterValidator(validate_const("customer.created")),
        ],
        pydantic.Field(alias="type"),
    ] = "customer.created"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattier import ProductPriceSeatTier, ProductPriceSeatTierTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ProductPriceSeatTiersTypedDict(TypedDict):
    r"""List of pricing tiers for seat-based pricing."""

    tiers: List[ProductPriceSeatTierTypedDict]
    r"""List of pricing tiers"""


class ProductPriceSeatTiers(BaseModel):
    r"""List of pricing tiers for seat-based pricing."""

    tiers: List[ProductPriceSeatTier]
    r"""List of pricing tiers"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


MetricsTotalsOrdersTypedDict = TypeAliasType(
    "MetricsTotalsOrdersTypedDict", Union[int, float]
)


MetricsTotalsOrders = TypeAliasType("MetricsTotalsOrders", Union[int, float])


MetricsTotalsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRevenueTypedDict", Union[int, float]
)


MetricsTotalsRevenue = TypeAliasType("MetricsTotalsRevenue", Union[int, float])


MetricsTotalsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsNetRevenue = TypeAliasType("MetricsTotalsNetRevenue", Union[int, float])


MetricsTotalsCumulativeRevenueTypedDict = TypeAliasType(
    "MetricsTotalsCumulativeRevenueTypedDict", Union[int, float]
)


MetricsTotalsCumulativeRevenue = TypeAliasType(
    "MetricsTotalsCumulativeRevenue", Union[int, float]
)


MetricsTotalsNetCumulativeRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNetCumulativeRevenueTypedDict", Union[int, float]
)


MetricsTotalsNetCumulativeRevenue = TypeAliasType(
    "MetricsTotalsNetCumulativeRevenue", Union[int, float]
)


MetricsTotalsAverageOrderValueTypedDict = TypeAliasType(
    "MetricsTotalsAverageOrderValueTypedDict", Union[int, float]
)


MetricsTotalsAverageOrderValue = TypeAliasType(
    "MetricsTotalsAverageOrderValue", Union[int, float]
)


MetricsTotalsNetAverageOrderValueTypedDict = TypeAliasType(
    "MetricsTotalsNetAverageOrderValueTypedDict", Union[int, float]
)


MetricsTotalsNetAverageOrderValue = TypeAliasType(
    "MetricsTotalsNetAverageOrderValue", Union[int, float]
)


MetricsTotalsOneTimeProductsTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProducts = TypeAliasType(
    "MetricsTotalsOneTimeProducts", Union[int, float]
)


MetricsTotalsOneTimeProductsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsRevenueTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProductsRevenue = TypeAliasType(
    "MetricsTotalsOneTimeProductsRevenue", Union[int, float]
)


MetricsTotalsOneTimeProductsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProductsNetRevenue = TypeAliasType(
    "MetricsTotalsOneTimeProductsNetRevenue", Union[int, float]
)


MetricsTotalsNewSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptions = TypeAliasType(
    "MetricsTotalsNewSubscriptions", Union[int, float]
)


MetricsTotalsNewSubscriptionsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsRevenueTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptionsRevenue = TypeAliasType(
    "MetricsTotalsNewSubscriptionsRevenue", Union[int, float]
)


MetricsTotalsNewSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptionsNetRevenue = TypeAliasType(
    "MetricsTotalsNewSubscriptionsNetRevenue", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptions = TypeAliasType(
    "MetricsTotalsRenewedSubscriptions", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsRevenueTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsRevenue = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsRevenue", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsNetRevenue = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsNetRevenue", Union[int, float]
)


MetricsTotalsActiveSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsActiveSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsActiveSubscriptions = TypeAliasType(
    "MetricsTotalsActiveSubscriptions", Union[int, float]
)


MetricsTotalsMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MetricsTotalsMonthlyRecurringRevenueTypedDict", Union[int, float]
)


MetricsTotalsMonthlyRecurringRevenue = TypeAliasType(
    "MetricsTotalsMonthlyRecurringRevenue", Union[int, float]
)


MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict", Union[int, float]
)


MetricsTotalsCommittedMonthlyRecurringRevenue = TypeAliasType(
    "MetricsTotalsCommittedMonthlyRecurringRevenue", Union[int, float]
)


MetricsTotalsCheckoutsTypedDict = TypeAliasType(
    "MetricsTotalsCheckoutsTypedDict", Union[int, float]
)


MetricsTotalsCheckouts = TypeAliasType("MetricsTotalsCheckouts", Union[int, float])


MetricsTotalsSucceededCheckoutsTypedDict = TypeAliasType(
    "MetricsTotalsSucceededCheckoutsTypedDict", Union[int, float]
)


MetricsTotalsSucceededCheckouts = TypeAliasType(
    "MetricsTotalsSucceededCheckouts", Union[int, float]
)


MetricsTotalsCheckoutsConversionTypedDict = TypeAliasType(
    "MetricsTotalsCheckoutsConversionTypedDict", Union[int, float]
)


MetricsTotalsCheckoutsConversion = TypeAliasType(
    "MetricsTotalsCheckoutsConversion", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptions = TypeAliasType(
    "MetricsTotalsCanceledSubscriptions", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsCustomerService = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsCustomerService", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsLowQualityTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsLowQualityTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsLowQuality = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsLowQuality", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsMissingFeatures = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsMissingFeatures", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsSwitchedService = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsSwitchedService", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooComplexTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooComplexTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooComplex = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooComplex", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooExpensive = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooExpensive", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsUnusedTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsUnusedTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsUnused = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsUnused", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsOtherTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsOtherTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsOther = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsOther", Union[int, float]
)


class MetricsTotalsTypedDict(TypedDict):
    orders: MetricsTotalsOrdersTypedDict
    revenue: MetricsTotalsRevenueTypedDict
    net_revenue: MetricsTotalsNetRevenueTypedDict
    cumulative_revenue: MetricsTotalsCumulativeRevenueTypedDict
    net_cumulative_revenue: MetricsTotalsNetCumulativeRevenueTypedDict
    average_order_value: MetricsTotalsAverageOrderValueTypedDict
    net_average_order_value: MetricsTotalsNetAverageOrderValueTypedDict
    one_time_products: MetricsTotalsOneTimeProductsTypedDict
    one_time_products_revenue: MetricsTotalsOneTimeProductsRevenueTypedDict
    one_time_products_net_revenue: MetricsTotalsOneTimeProductsNetRevenueTypedDict
    new_subscriptions: MetricsTotalsNewSubscriptionsTypedDict
    new_subscriptions_revenue: MetricsTotalsNewSubscriptionsRevenueTypedDict
    new_subscriptions_net_revenue: MetricsTotalsNewSubscriptionsNetRevenueTypedDict
    renewed_subscriptions: MetricsTotalsRenewedSubscriptionsTypedDict
    renewed_subscriptions_revenue: MetricsTotalsRenewedSubscriptionsRevenueTypedDict
    renewed_subscriptions_net_revenue: (
        MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict
    )
    active_subscriptions: MetricsTotalsActiveSubscriptionsTypedDict
    monthly_recurring_revenue: MetricsTotalsMonthlyRecurringRevenueTypedDict
    committed_monthly_recurring_revenue: (
        MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict
    )
    checkouts: MetricsTotalsCheckoutsTypedDict
    succeeded_checkouts: MetricsTotalsSucceededCheckoutsTypedDict
    checkouts_conversion: MetricsTotalsCheckoutsConversionTypedDict
    canceled_subscriptions: MetricsTotalsCanceledSubscriptionsTypedDict
    canceled_subscriptions_customer_service: (
        MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict
    )
    canceled_subscriptions_low_quality: (
        MetricsTotalsCanceledSubscriptionsLowQualityTypedDict
    )
    canceled_subscriptions_missing_features: (
        MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict
    )
    canceled_subscriptions_switched_service: (
        MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict
    )
    canceled_subscriptions_too_complex: (
        MetricsTotalsCanceledSubscriptionsTooComplexTypedDict
    )
    canceled_subscriptions_too_expensive: (
        MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict
    )
    canceled_subscriptions_unused: MetricsTotalsCanceledSubscriptionsUnusedTypedDict
    canceled_subscriptions_other: MetricsTotalsCanceledSubscriptionsOtherTypedDict


class MetricsTotals(BaseModel):
    orders: MetricsTotalsOrders

    revenue: MetricsTotalsRevenue

    net_revenue: MetricsTotalsNetRevenue

    cumulative_revenue: MetricsTotalsCumulativeRevenue

    net_cumulative_revenue: MetricsTotalsNetCumulativeRevenue

    average_order_value: MetricsTotalsAverageOrderValue

    net_average_order_value: MetricsTotalsNetAverageOrderValue

    one_time_products: MetricsTotalsOneTimeProducts

    one_time_products_revenue: MetricsTotalsOneTimeProductsRevenue

    one_time_products_net_revenue: MetricsTotalsOneTimeProductsNetRevenue

    new_subscriptions: MetricsTotalsNewSubscriptions

    new_subscriptions_revenue: MetricsTotalsNewSubscriptionsRevenue

    new_subscriptions_net_revenue: MetricsTotalsNewSubscriptionsNetRevenue

    renewed_subscriptions: MetricsTotalsRenewedSubscriptions

    renewed_subscriptions_revenue: MetricsTotalsRenewedSubscriptionsRevenue

    renewed_subscriptions_net_revenue: MetricsTotalsRenewedSubscriptionsNetRevenue

    active_subscriptions: MetricsTotalsActiveSubscriptions

    monthly_recurring_revenue: MetricsTotalsMonthlyRecurringRevenue

    committed_monthly_recurring_revenue: MetricsTotalsCommittedMonthlyRecurringRevenue

    checkouts: MetricsTotalsCheckouts

    succeeded_checkouts: MetricsTotalsSucceededCheckouts

    checkouts_conversion: MetricsTotalsCheckoutsConversion

    canceled_subscriptions: MetricsTotalsCanceledSubscriptions

    canceled_subscriptions_customer_service: (
        MetricsTotalsCanceledSubscriptionsCustomerService
    )

    canceled_subscriptions_low_quality: MetricsTotalsCanceledSubscriptionsLowQuality

    canceled_subscriptions_missing_features: (
        MetricsTotalsCanceledSubscriptionsMissingFeatures
    )

    canceled_subscriptions_switched_service: (
        MetricsTotalsCanceledSubscriptionsSwitchedService
    )

    canceled_subscriptions_too_complex: MetricsTotalsCanceledSubscriptionsTooComplex

    canceled_subscriptions_too_expensive: MetricsTotalsCanceledSubscriptionsTooExpensive

    canceled_subscriptions_unused: MetricsTotalsCanceledSubscriptionsUnused

    canceled_subscriptions_other: MetricsTotalsCanceledSubscriptionsOther



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebTokenRequestSubType(str, Enum):
    USER = "user"
    ORGANIZATION = "organization"


class WebTokenRequestTypedDict(TypedDict):
    client_id: str
    client_secret: str
    session_token: str
    grant_type: Literal["web"]
    sub_type: NotRequired[WebTokenRequestSubType]
    sub: NotRequired[Nullable[str]]
    scope: NotRequired[Nullable[str]]


class WebTokenRequest(BaseModel):
    client_id: Annotated[str, FieldMetadata(form=True)]

    client_secret: Annotated[str, FieldMetadata(form=True)]

    session_token: Annotated[str, FieldMetadata(form=True)]

    GRANT_TYPE: Annotated[
        Annotated[Literal["web"], AfterValidator(validate_const("web"))],
        pydantic.Field(alias="grant_type"),
        FieldMetadata(form=True),
    ] = "web"

    sub_type: Annotated[Optional[WebTokenRequestSubType], FieldMetadata(form=True)] = (
        WebTokenRequestSubType.USER
    )

    sub: Annotated[OptionalNullable[str], FieldMetadata(form=True)] = UNSET

    scope: Annotated[OptionalNullable[str], FieldMetadata(form=True)] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["sub_type", "sub", "scope"]
        nullable_fields = ["sub", "scope"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
import importlib
from polar_sdk import models, utils
from polar_sdk._hooks import SDKHooks
from polar_sdk.types import OptionalNullable, UNSET
import sys
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from polar_sdk.benefit_grants import BenefitGrants
    from polar_sdk.benefits import Benefits
    from polar_sdk.checkout_links import CheckoutLinks
    from polar_sdk.checkouts import Checkouts
    from polar_sdk.custom_fields import CustomFields
    from polar_sdk.customer_meters import CustomerMeters
    from polar_sdk.customer_portal import CustomerPortal
    from polar_sdk.customer_sessions import CustomerSessions
    from polar_sdk.customers import Customers
    from polar_sdk.discounts import Discounts
    from polar_sdk.events import Events
    from polar_sdk.files import Files
    from polar_sdk.license_keys import LicenseKeys
    from polar_sdk.meters import Meters
    from polar_sdk.metrics_sdk import MetricsSDK
    from polar_sdk.oauth2 import Oauth2
    from polar_sdk.orders import Orders
    from polar_sdk.organizations import Organizations
    from polar_sdk.payments import Payments
    from polar_sdk.products import Products
    from polar_sdk.refunds import Refunds
    from polar_sdk.subscriptions import Subscriptions
    from polar_sdk.webhooks import Webhooks


class Polar(BaseSDK):
    r"""Polar API: Polar HTTP and Webhooks API

    Read the docs at https://polar.sh/docs/api-reference
    """

    organizations: "Organizations"
    subscriptions: "Subscriptions"
    oauth2: "Oauth2"
    benefits: "Benefits"
    benefit_grants: "BenefitGrants"
    webhooks: "Webhooks"
    products: "Products"
    orders: "Orders"
    refunds: "Refunds"
    checkouts: "Checkouts"
    files: "Files"
    metrics: "MetricsSDK"
    license_keys: "LicenseKeys"
    checkout_links: "CheckoutLinks"
    custom_fields: "CustomFields"
    discounts: "Discounts"
    customers: "Customers"
    customer_portal: "CustomerPortal"
    customer_sessions: "CustomerSessions"
    events: "Events"
    meters: "Meters"
    customer_meters: "CustomerMeters"
    payments: "Payments"
    _sub_sdk_map = {
        "organizations": ("polar_sdk.organizations", "Organizations"),
        "subscriptions": ("polar_sdk.subscriptions", "Subscriptions"),
        "oauth2": ("polar_sdk.oauth2", "Oauth2"),
        "benefits": ("polar_sdk.benefits", "Benefits"),
        "benefit_grants": ("polar_sdk.benefit_grants", "BenefitGrants"),
        "webhooks": ("polar_sdk.webhooks", "Webhooks"),
        "products": ("polar_sdk.products", "Products"),
        "orders": ("polar_sdk.orders", "Orders"),
        "refunds": ("polar_sdk.refunds", "Refunds"),
        "checkouts": ("polar_sdk.checkouts", "Checkouts"),
        "files": ("polar_sdk.files", "Files"),
        "metrics": ("polar_sdk.metrics_sdk", "MetricsSDK"),
        "license_keys": ("polar_sdk.license_keys", "LicenseKeys"),
        "checkout_links": ("polar_sdk.checkout_links", "CheckoutLinks"),
        "custom_fields": ("polar_sdk.custom_fields", "CustomFields"),
        "discounts": ("polar_sdk.discounts", "Discounts"),
        "customers": ("polar_sdk.customers", "Customers"),
        "customer_portal": ("polar_sdk.customer_portal", "CustomerPortal"),
        "customer_sessions": ("polar_sdk.customer_sessions", "CustomerSessions"),
        "events": ("polar_sdk.events", "Events"),
        "meters": ("polar_sdk.meters", "Meters"),
        "customer_meters": ("polar_sdk.customer_meters", "CustomerMeters"),
        "payments": ("polar_sdk.payments", "Payments"),
    }

    def __init__(
        self,
        access_token: Optional[
            Union[Optional[str], Callable[[], Optional[str]]]
        ] = None,
        server: Optional[str] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param access_token: The access_token required for authentication
        :param server: The server by name to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(access_token):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(access_token=access_token())
        else:
            security = models.Security(access_token=access_token)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server=server,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerBalanceTypedDict(TypedDict):
    r"""Customer balance information."""

    balance: int
    r"""Customer balance in cents. Positive values represent credit (customer is owed money), negative values represent debit (customer owes money)."""
    currency: str
    r"""The currency code (ISO 4217) for the balance amount."""


class CustomerBalance(BaseModel):
    r"""Customer balance information."""

    balance: int
    r"""Customer balance in cents. Positive values represent credit (customer is owed money), negative values represent debit (customer owes money)."""

    currency: str
    r"""The currency code (ISO 4217) for the balance amount."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardpayment import CardPayment, CardPaymentTypedDict
from .genericpayment import GenericPayment, GenericPaymentTypedDict
from typing import Union
from typing_extensions import TypeAliasType


PaymentTypedDict = TypeAliasType(
    "PaymentTypedDict", Union[GenericPaymentTypedDict, CardPaymentTypedDict]
)


Payment = TypeAliasType("Payment", Union[GenericPayment, CardPayment])



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Dict, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


LicenseKeyActivateConditionsTypedDict = TypeAliasType(
    "LicenseKeyActivateConditionsTypedDict", Union[str, int, float, bool]
)


LicenseKeyActivateConditions = TypeAliasType(
    "LicenseKeyActivateConditions", Union[str, int, float, bool]
)


LicenseKeyActivateMetaTypedDict = TypeAliasType(
    "LicenseKeyActivateMetaTypedDict", Union[str, int, float, bool]
)


LicenseKeyActivateMeta = TypeAliasType(
    "LicenseKeyActivateMeta", Union[str, int, float, bool]
)


class LicenseKeyActivateTypedDict(TypedDict):
    key: str
    organization_id: str
    label: str
    conditions: NotRequired[Dict[str, LicenseKeyActivateConditionsTypedDict]]
    r"""Key-value object allowing you to set conditions that must match when validating the license key.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    meta: NotRequired[Dict[str, LicenseKeyActivateMetaTypedDict]]
    r"""Key-value object allowing you to store additional information about the activation

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """


class LicenseKeyActivate(BaseModel):
    key: str

    organization_id: str

    label: str

    conditions: Optional[Dict[str, LicenseKeyActivateConditions]] = None
    r"""Key-value object allowing you to set conditions that must match when validating the license key.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    meta: Optional[Dict[str, LicenseKeyActivateMeta]] = None
    r"""Key-value object allowing you to store additional information about the activation

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrganizationCustomerEmailSettingsTypedDict(TypedDict):
    order_confirmation: bool
    subscription_cancellation: bool
    subscription_confirmation: bool
    subscription_cycled: bool
    subscription_past_due: bool
    subscription_revoked: bool
    subscription_uncanceled: bool
    subscription_updated: bool


class OrganizationCustomerEmailSettings(BaseModel):
    order_confirmation: bool

    subscription_cancellation: bool

    subscription_confirmation: bool

    subscription_cycled: bool

    subscription_past_due: bool

    subscription_revoked: bool

    subscription_uncanceled: bool

    subscription_updated: bool
